﻿/*
====== READ THIS ========
This code was created as a part of the seminar on 
flash game programing at the University of Munich (Ludwig-
Maximlians Universität.

The code may be used for educational purposes only. All
rights reserverd by the author.

Author: Max Maurer
E-Mail: max.maurer@ifi.lmu.de
(c) 2011
If you have any questions don't hasitate to contact me.
=========================
*/
package {
	import flash.display.MovieClip;
	import flash.events.Event;
	
	public class Main extends MovieClip {
		private var balls:Array;
		private var cueBall:Ball;
		private var lastCueX:Number;
		private var lastCueY:Number;
		private const SHOOT_DISTANCE:Number=30;
		private var gameMode:int;
		private const MODE_AIM:int = 1;
		private const MODE_SHOOT:int = 2;
		private var cue:Cue;
		public function Main():void {
		}
		
		public function init():void {
			resetGame();
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);	
		}
		
		private function resetGame():void {
			if (balls!=null) {
				// clean up before running a new game
				for (var i:int =0;i<balls.length;i++) {
					removeChild(balls[i]);
				}
				removeChild(cue);
			}
			balls = new Array();
			cueBall = createBall(3,table.x+(table.face.width)*9/12,table.y+(table.face.height)/2);
			setupDeck(table.x+65,table.y+65);
			cue = new Cue();
			addChild(cue);
			gameMode=MODE_AIM;
		}
		
		
		private function setupDeck(sX:Number, sY:Number):void {
			// create ball deck starting at sX and sY
			var diam:Number = cueBall.diam;
			// first row
			createBall(1,sX,sY);
			createBall(2,sX,sY+diam);
			createBall(1,sX,sY+diam*2);
			createBall(1,sX,sY+diam*3);
			createBall(2,sX,sY+diam*4);
			// second row
			sY+=diam/2;
			sX+=diam;
			createBall(2,sX,sY);
			createBall(1,sX,sY+diam);
			createBall(2,sX,sY+diam*2);
			createBall(1,sX,sY+diam*3);
			// third row
			sY+=diam/2;
			sX+=diam;
			createBall(1,sX,sY);
			createBall(4,sX,sY+diam);
			createBall(2,sX,sY+diam*2);
			// fourth row
			sY+=diam/2;
			sX+=diam;
			createBall(2,sX,sY);
			createBall(1,sX,sY+diam);
			// fifth row
			sY+=diam/2;
			sX+=diam;
			createBall(2,sX,sY);
		}
		
		private function createBall(color:int,bx:Number,by:Number):Ball {
			var b:Ball = new Ball(color);
			b.x = bx;
			b.y = by;
			balls.push(b);
			addChild(b);
			return b;
		}
		
		private function enterFrameHandler(e:Event):void {
			if (gameMode==MODE_SHOOT) {
				if (!moveBalls()) {
					cue.visible=true;
					gameMode=MODE_AIM;
					scoreControl.nextPlayer();
				}
				checkHoles();
			}
			if (gameMode==MODE_AIM) {
				aim();
			}
			checkCollision();
		}
		
		private function checkHoles():void {
			for (var i:int =0;i<balls.length;i++) {
				var b:Ball = Ball(balls[i]);
				if (table.holes.hitTestPoint(b.x,b.y,true)) {
					var result:String = scoreControl.putBall(b);
					if (result=="foul") {
						cueBall.vx = cueBall.vy = 0;
						cueBall.x = table.x+(table.face.width)*9/12;
						cueBall.y = table.y+(table.face.height)/2;
					} else if (result=="lost" || result=="won") {
						resetGame();
					} else {
						removeChild(b);
						// remove the ball by replacing it with the last one, popping this one and retrying on that index again
						balls[i] = balls[balls.length-1];
						balls.pop();
						i--;
					}
				}
			}
		}
		
		function aim():void {
			var dx:Number = cueBall.x-mouseX;
			var dy:Number = cueBall.y-mouseY;
			var angle:Number = Math.atan2(dy, dx);
			cue.rotation = angle*180/Math.PI;
			cue.x = mouseX;
			cue.y = mouseY;
			if (Math.sqrt(dx*dx+dy*dy)<SHOOT_DISTANCE) {
				gameMode=MODE_SHOOT;
				scoreControl.setStatus("Please wait...");
				cueBall.vx = cue.x-lastCueX;
				cueBall.vy = cue.y-lastCueY;
				cue.visible=false;
			}
			lastCueX = cue.x;
			lastCueY = cue.y;			
		}
		
		function moveBalls():Boolean {
			var ballMoved:Boolean=false;
			for (var i:int =0;i<balls.length;i++) {
				var b:Ball = Ball(balls[i]);
				if (b.move(table.y,table.x+table.face.width,table.y+table.face.height,table.x)) {
					ballMoved=true;
				}
			}
			return ballMoved;
		}
		
		function checkCollision() {
			for (var i=0; i<balls.length-1; i++) {
				var b1:Ball = balls[i];
				for (var j=i+1; j<balls.length; j++) {
					var b2:Ball = balls[j];
					b1.checkCollission(b2);
				}
			}
		}
}
}